<?php

namespace CountriesStates\CountriesState\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use CountriesStates\CountriesState\Http\Requests\CountriesRequest;
use CountriesStates\CountriesState\Interfaces\CountriesRepositoryInterface;
use CountriesStates\CountriesState\Models\Countries;

/**
 * Resource controller class for countries.
 */
class CountriesResourceController extends BaseController
{

    /**
     * Initialize countries resource controller.
     *
     * @param type CountriesRepositoryInterface $countries
     *
     * @return null
     */
    public function __construct(CountriesRepositoryInterface $countries)
    {
        parent::__construct();
        $this->repository = $countries;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\CountriesStates\CountriesState\Repositories\Criteria\CountriesResourceCriteria::class);
    }

    /**
     * Display a list of countries.
     *
     * @return Response
     */
    public function index(CountriesRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\CountriesStates\CountriesState\Repositories\Presenter\CountriesPresenter::class)
                ->$function();
        }

        $countries = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('countries_state::countries.names'))
            ->view('countries_state::countries.index', true)
            ->data(compact('countries', 'view'))
            ->output();
    }

    /**
     * Display countries.
     *
     * @param Request $request
     * @param Model   $countries
     *
     * @return Response
     */
    public function show(CountriesRequest $request, Countries $countries)
    {

        if ($countries->exists) {
            $view = 'countries_state::countries.show';
        } else {
            $view = 'countries_state::countries.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('countries_state::countries.name'))
            ->data(compact('countries'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new countries.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CountriesRequest $request)
    {

        $countries = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('countries_state::countries.name')) 
            ->view('countries_state::countries.create', true) 
            ->data(compact('countries'))
            ->output();
    }

    /**
     * Create new countries.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CountriesRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $countries                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('countries_state::countries.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('countries_state/countries/' . $countries->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/countries_state/countries'))
                ->redirect();
        }

    }

    /**
     * Show countries for editing.
     *
     * @param Request $request
     * @param Model   $countries
     *
     * @return Response
     */
    public function edit(CountriesRequest $request, Countries $countries)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('countries_state::countries.name'))
            ->view('countries_state::countries.edit', true)
            ->data(compact('countries'))
            ->output();
    }

    /**
     * Update the countries.
     *
     * @param Request $request
     * @param Model   $countries
     *
     * @return Response
     */
    public function update(CountriesRequest $request, Countries $countries)
    {
        try {
            $attributes = $request->all();

            $countries->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('countries_state::countries.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('countries_state/countries/' . $countries->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('countries_state/countries/' . $countries->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the countries.
     *
     * @param Model   $countries
     *
     * @return Response
     */
    public function destroy(CountriesRequest $request, Countries $countries)
    {
        try {

            $countries->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('countries_state::countries.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('countries_state/countries/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('countries_state/countries/' . $countries->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple countries.
     *
     * @param Model   $countries
     *
     * @return Response
     */
    public function delete(CountriesRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('countries_state::countries.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('countries_state/countries'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/countries_state/countries'))
                ->redirect();
        }

    }

    /**
     * Restore deleted countries.
     *
     * @param Model   $countries
     *
     * @return Response
     */
    public function restore(CountriesRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('countries_state::countries.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/countries_state/countries'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/countries_state/countries/'))
                ->redirect();
        }

    }

}
